/**
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 * 
 * This software is published by the license of CPU-OS License，you can use and 
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us 
 * by email <support_os@cpu-os.ac.cn>  
 * 
**/

#ifndef __CDOS_LIST_H__
#define __CDOS_LIST_H__

#include <core/cdos-def.h>

CDOS_BEGIN_DECLS

typedef struct _CdosList CdosList;
typedef struct _CdosListNode CdosListNode;

struct _CdosListNode {
    CdosListNode* prev;
    CdosPointer data;
    CdosListNode* next;
};

typedef void (*CdosListForeachFunc)(CdosUInt pos, CdosPointer data, CdosPointer user_data);
typedef int (*CdosListCompareFunc)(CdosConstPointer data1, CdosConstPointer data2);
typedef int (*CdosListCompareFuncWithData)(CdosConstPointer data1, CdosConstPointer data2, CdosPointer user_data);

CDOS_EXPORT CdosList* cdos_list_new();

CDOS_EXPORT CdosList* cdos_list_append(CdosList* list, CdosPointer data);
CDOS_EXPORT CdosList* cdos_list_prepend(CdosList* list, CdosPointer data);
CDOS_EXPORT CdosList* cdos_list_insert(CdosList* list, CdosPointer data, int position);

CDOS_EXPORT CdosListNode* cdos_list_first(CdosList* list);
CDOS_EXPORT CdosListNode* cdos_list_last(CdosList* list);
CDOS_EXPORT CdosListNode* cdos_list_nth(CdosList* list, int index);
CDOS_EXPORT CdosPointer* cdos_list_nth_data(CdosList* list, int index);


CDOS_EXPORT unsigned int cdos_list_length(CdosList* list);

CDOS_EXPORT void cdos_list_foreach(CdosList* list, CdosListForeachFunc func, CdosPointer user_data);

// TODO: 排序功能暂未实现，需要先确定需要使用的排序算法。
CDOS_EXPORT void cdos_list_sort(CdosList* list, CdosListCompareFunc func);
CDOS_EXPORT void cdos_list_sort_with_data(CdosList* list, CdosListCompareFuncWithData func, CdosPointer user_data);



CDOS_EXPORT void cdos_list_delete(CdosList** list);

CDOS_END_DECLS

#endif // __CDOS_LIST_H__