/**
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 * 
 * This software is published by the license of CPU-OS License，you can use and 
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us 
 * by email <support_os@cpu-os.ac.cn>  
 * 
**/

#ifndef __CDOS_LOG_H__

#define __CDOS_LOG_H__

#include <core/cdos-def.h>
#include <stdarg.h>

CDOS_BEGIN_DECLS

typedef enum {
    /**
     * 最低优先级别。
     *
     * 主要用于打印调试日志
     */
    CDOS_LOGLV_INFO = 0,

    /**
     * 调试级别日志。
     *
     * 主要用于打印调试日志
     */
    CDOS_LOGLV_DEBUG,

    /**
     * 警告级别日志。
     *
     * 主要用于打印警告信息
     */
    CDOS_LOGLV_WARNING,

    /**
     * 错误级别日志。
     *
     * 主要用于打印错误信息或可能引起系统异常的错误信息。
     *
     * 默认开启，无法关闭。
     */
    CDOS_LOGLV_ERROR,

    /**
     * 最高优先级别日志。
     *
     * 主要用于打印致命错误信息或会引起系统异常的错误信息。
     */
    CDOS_LOGLV_FATAL
} CdosLogLevel;

CDOS_EXPORT void cdos_api_log_Common(const char* logtag, CdosLogLevel level, const char* file, int line, const char* func, const char* format, ...) CDOS_FORMAT_FUNCTION(6, 7);

#define LOGI(...) cdos_api_log_Common(TAG, CDOS_LOGLV_INFO, __FILE__, __LINE__, __func__, __VA_ARGS__);
#define LOGD(...) cdos_api_log_Common(TAG, CDOS_LOGLV_DEBUG, __FILE__, __LINE__, __func__, __VA_ARGS__);
#define LOGW(...) cdos_api_log_Common(TAG, CDOS_LOGLV_WARNING, __FILE__, __LINE__, __func__, __VA_ARGS__);
#define LOGE(...) cdos_api_log_Common(TAG, CDOS_LOGLV_ERROR, __FILE__, __LINE__, __func__, __VA_ARGS__);
#define LOGF(...) cdos_api_log_Common(TAG, CDOS_LOGLV_FATAL, __FILE__, __LINE__, __func__, __VA_ARGS__);

CDOS_END_DECLS

#endif // __CDOS_LOG_H__