/**
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 * 
 * Based on code from notify.h in libnotify.
 * Copyright (C) 2004-2006 Christian Hammond.
 * Copyright (C) 2010 Red Hat, Inc.
 * 
 * This software is published by the license of CPU-OS License，you can use and 
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us 
 * by email <support_os@cpu-os.ac.cn>  
 * 
**/

/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
 *
 * Copyright (C) 2004-2006 Christian Hammond
 * Copyright (C) 2010 Red Hat, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#ifndef _LIBNOTIFY_NOTIFY_H_
#define _LIBNOTIFY_NOTIFY_H_

#include <glib.h>

#include "cdos-notification.h"


G_BEGIN_DECLS

/**
 * cdos_notify_init:
 * @app_name: The name of the application initializing libcdosnotify.
 *
 * Initialized libcdosnotify. This must be called before any other functions.
 *
 * Returns: %TRUE if successful, or %FALSE on error.
 */
gboolean        cdos_notify_init (const char *app_name);

/**
 * cdos_notify_uninit:
 *
 * Uninitialized libcdosnotify.
 *
 * This should be called when the program no longer needs libcdosnotify for
 * the rest of its lifecycle, typically just before exitting.
 */
void            cdos_notify_uninit (void);

/**
 * cdos_notify_is_initted:
 *
 * Gets whether or not libcdosnotify is initialized.
 *
 * Returns: %TRUE if libcdosnotify is initialized, or %FALSE otherwise.
 */
gboolean        cdos_notify_is_initted (void);

/**
 * cdos_notify_get_app_name:
 *
 * Gets the application name registered.
 *
 * Returns: The registered application name, passed to cdos_notify_init().
 */
const char     *cdos_notify_get_app_name (void);

/**
 * cdos_notify_set_app_name:
 * @app_name: The name of the application
 *
 * Sets the application name.
 *
 */
void            cdos_notify_set_app_name (const char *app_name);

/**
 * cdos_notify_get_server_caps:
 *
 * Synchronously queries the server for its capabilities and returns them in a #GList.
 *
 * Returns: (transfer full) (element-type utf8): a #GList of server capability strings. Free
 *   the list elements with g_free() and the list itself with g_list_free().
 */
GList          *cdos_notify_get_server_caps (void);

/**
 * cdos_notify_get_server_info:
 * @ret_name: (out) (allow-none) (transfer full): a location to store the server name, or %NULL
 * @ret_vendor: (out) (allow-none) (transfer full): a location to store the server vendor, or %NULL
 * @ret_version: (out) (allow-none) (transfer full): a location to store the server version, or %NULL
 * @ret_spec_version: (out) (allow-none) (transfer full): a location to store the version the service is compliant with, or %NULL
 *
 * Synchronously queries the server for its information, specifically, the name, vendor,
 * server version, and the version of the notifications specification that it
 * is compliant with.
 *
 * Returns: %TRUE if successful, and the variables passed will be set, %FALSE
 *          on error. The returned strings must be freed with g_free
 */
gboolean        cdos_notify_get_server_info (char **ret_name,
                                        char **ret_vendor,
                                        char **ret_version,
                                        char **ret_spec_version);

/**
 * cdos_notify_add_notify_blacklist:
 * @app_name: (allow-none): The name of the application.
 * @level: The level id.
 *
 * Synchronously add application to blacklist.
 *
 */
void cdos_notify_add_notify_blacklist (gchar *app_name, guint level);

/**
 * cdos_notify_add_notify_blacklist_unblock_all:
 *
 * Synchronously add application to blacklist.
 *
 */
void cdos_notify_add_notify_blacklist_unblock_all ();

/**
 * cdos_notify_add_notify_blacklist_block_all:
 *
 * Synchronously add application to blacklist.
 *
 */
void cdos_notify_add_notify_blacklist_block_all ();

/**
 * cdos_notify_add_notify_blacklist_block:
 * @app_name: The name of the application.
 *
 * Synchronously add application to blacklist.
 *
 */
void cdos_notify_add_notify_blacklist_block (gchar *app_name);

/**
 * cdos_notify_add_notify_blacklist_clean:
 * @app_name: The name of the application.
 *
 * Synchronously add application to blacklist.
 *
 */
void cdos_notify_add_notify_blacklist_clean (gchar *app_name);

/**
 * cdos_notify_add_notify_blacklist_dnd:
 * @app_name: The name of the application.
 *
 * Synchronously add application to blacklist.
 *
 */
void cdos_notify_add_notify_blacklist_dnd (gchar *app_name);

/**
 * cdos_notify_add_notify_blacklist_silent:
 * @app_name: The name of the application.
 *
 * Synchronously add application to blacklist.
 *
 */
void cdos_notify_add_notify_blacklist_silent (gchar *app_name);

/**
 * cdos_notify_add_notify_blacklist_clear:
 * @app_name: The name of the application.
 *
 * Synchronously add application to blacklist.
 *
 */
void cdos_notify_add_notify_blacklist_clear (gchar *app_name);
G_END_DECLS

#endif /* _LIBNOTIFY_NOTIFY_H_ */
